from django.conf import settings
from helpers import check_allow_for_user


class ImpersonateMiddleware(object):
    def process_request(self, request):
        if request.user.is_authenticated() and \
           '_impersonate' in request.session:
            new_user = request.session['_impersonate']
            if check_allow_for_user(request.user, new_user):
                request.user = new_user
                request.user.is_impersonate = True



from django.conf import settings
from django.db.models import Q
from django.template import RequestContext
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404, redirect, render_to_response
from decorators import allowed_user_required
from helpers import get_redir_path, get_paginator, check_allow_for_user


@allowed_user_required
def impersonate(request, uid):
    ''' Takes in the UID of the user to impersonate.
        View will fetch the User instance and store it 
        in the request.session under the '_impersonate' key.
        
        The middleware will then pick up on it and adjust the 
        request object as needed.
    '''
    new_user = get_object_or_404(User, pk=uid)
    if check_allow_for_user(request.user, new_user):
        request.session['_impersonate'] = new_user
        request.session.modified = True  # Let's make sure...
    return redirect(get_redir_path())


def stop_impersonate(request):
    ''' Remove the impersonation object from the session
    '''
    if '_impersonate' in request.session:
        del request.session['_impersonate']
        request.session.modified = True
    return redirect(get_redir_path())


@allowed_user_required
def list_users(request, template):
    ''' List all users in the system.
        Will add 4 items to the context.
          * users - queryset of all users
          * paginator - Django Paginator instance
          * page - Current page of objects (from Paginator)
          * page_number - Current page number, defaults to 1
    '''
    users = User.objects.all()
    paginator, page, page_number = get_paginator(request, users)

    return render_to_response(template, {
        'users': users,
        'paginator': paginator,
        'page': page,
        'page_number': page_number,
    }, context_instance=RequestContext(request))


@allowed_user_required
def search_users(request, template):
    ''' Simple search through the users.
        Will add 5 items to the context.
          * users - All users that match the query passed.
          * paginator - Django Paginator instance
          * page - Current page of objects (from Paginator)
          * page_number - Current page number, defaults to 1
          * query - The search query that was entered
    '''
    query = request.GET.get('q', '')
    search_q = Q(username__icontains=query) | \
               Q(first_name__icontains=query) | \
               Q(last_name__icontains=query) | \
               Q(email__icontains=query)
    users = User.objects.filter(search_q)
    paginator, page, page_number = get_paginator(request, users)

    return render_to_response(template, {
        'users': users,
        'paginator': paginator,
        'page': page,
        'page_number': page_number,
        'query': query,
    }, context_instance=RequestContext(request))



from django.conf import settings
from django.shortcuts import redirect
from django.utils.http import urlquote
from django.contrib.auth import REDIRECT_FIELD_NAME
from helpers import get_redir_path, check_allow_staff


def allowed_user_required(view_func):
    def _checkuser(request, *args, **kwargs):
        allow_staff = check_allow_staff()
        if not request.user.is_authenticated():
            return redirect('%s?%s=%s' % (
                settings.LOGIN_URL,
                REDIRECT_FIELD_NAME,
                urlquote(request.get_full_path()),
            ))

        if getattr(request.user, 'is_impersonate', False):
            # Do not allow an impersonated session to use the 
            # impersonate views.
            return redirect(get_redir_path())

        if not request.user.is_superuser:
            if not request.user.is_staff or not allow_staff:
                return redirect(get_redir_path())

        return view_func(request, *args, **kwargs)
    return _checkuser



from django.conf import settings
from django.core.paginator import Paginator, EmptyPage

 
def get_redir_path():        
    return getattr(
        settings,
        'IMPERSONATE_REDIRECT_URL',
        getattr(settings, 'LOGIN_REDIRECT_URL', '/'),
    )       
        
    
def get_paginator(request, qs):
    try:
        page_number = int(request.GET.get('page', 1))
    except ValueError:
        page_number = 1

    paginator = Paginator(
        qs,
        int(getattr(settings, 'IMPERSONATE_PAGINATE_COUNT', 20)),
    )   
    try:
        page = paginator.page(page_number)
    except EmptyPage:
        page = None

    return (paginator, page, page_number)


def check_allow_staff():
    return (not getattr(settings, 'IMPERSONATE_REQUIRE_SUPERUSER', False))


def check_allow_for_user(start_user, end_user):
    if not start_user.is_superuser:
        allow_staff = check_allow_staff()
        if start_user.is_staff and allow_staff:
            if not end_user.is_superuser:
                return True
    else:
        return True

    return False
